import Cookie

from bn import AttributeDict, str_dict
from conversionkit import Conversion

#
# Translator Service
#

def make_i18n_config(flow, name):

    import os

    from recordconvert import toRecord
    from stringconvert import unicodeToUnicode
    from configconvert import existingDirectory
 
    return toRecord(
        missing_defaults = dict(
            domain=u"messages", 
            locale_dir=os.path.join(flow.config.app.app_dir, 'i18n'),
        ),
        empty_errors = "The option '%s.%%(key)s' cannot be empty"%(name,),
        converters=dict(
            default_language = unicodeToUnicode(),
            locale_dir = existingDirectory(),
            domain = unicodeToUnicode(),
        )
    )

def gettext_noop(value):
    """Mark a string for translation without translating it. Returns
    value.

    Used for global strings, e.g.::

        foo = flow.i18n.N_('Hello')

        class Bar:
            def __init__(self):
                self.local_foo = flow.i18n._(foo)

        flow.i18n..set_lang('fr')
        assert Bar().local_foo == 'Bonjour'
        flow.i18n.set_lang('es')
        assert Bar().local_foo == 'Hola'
        assert foo == 'Hello'
    """
    return value

class DomainService(object):
    """
    i18n service.

    Looks for message catalogs relative to the localdir:

    localedir/language/LC_MESSAGES/domain.mo

    pybabel extract myapp -o myapp/i18n/messages.pot
    pybabel init -l en -i myapp/i18n/messages.pot -d myapp/i18n
    pybabel compile -l en -d myapp/i18n -f
    pybabel update -l en -d myapp/i18n -i myapp/i18n/messages.pot

    For example, to use the ``messages.mo`` file in ``myapp/i18n/fr/LC_MESSAGES/``
    you would do this:

        _get_translator('message', languages=['fr'], localedir='myapp/i18n')

    This is wrapped up in this API:

        flow.i18n.set(languages=[], domain='messages')

    If no ``domain`` is specified ``self.domain`` is used, defaulting to
    ``'messages'``. If no ``languages`` are specified a NullTranslations is used
    which leaves all strings as they are in the code. You can specify a string,
    instead of a list for languages, if you don't want the fallbacks.

    This service therefore takes the defaults specified 

    ToDo:
        * Investigate whether caching the translators is actually useful or not.
    """
    def __init__(self, locale_dir, domain='messages', default_language=None):
        self.default_language = default_language
        self.domain = domain
        self.localedir = locale_dir

    @staticmethod
    def create(flow, name, config=None):
        if config is None:
            config = MailService.config(flow, name)
        return DomainService(**str_dict(config))

    @staticmethod
    def config(flow, name):
        from configconvert import handle_option_error, handle_section_error
        if not flow.config.option_group.has_key(name):
            raise handle_section_error(
                flow, 
                name, 
                "'%s.default_language'"%(name),
            )
        converter = make_i18n_config(flow, name)
        conversion = Conversion(
            flow.config.option_group[name]
        ).perform(converter)
        if not conversion.successful:
            handle_option_error(conversion, name)
        else:
            flow.config[name] = conversion.result
        return flow.config[name]

    def start(self, flow, name):
        flow[name] = AttributeDict()

        def set_lang(languages=None, domain=None):
            if not languages:
                from gettext import NullTranslations
                flow[name]['translator'] = NullTranslations()
                flow[name]['languages'] = []
            else:
                from gettext import translation
                #gettext.translation(domain[, localedir[, languages[, class_[, fallback[, codeset]]]]])
                if not isinstance(languages, (list, tuple)): 
                    languages = [languages]
                t = translation(
                    domain=domain or self.domain,
                    localedir=self.localedir,
                    languages = languages,
                )
                flow[name]['translator'] = t
                flow[name]['languages'] = languages

        def info():
            flow[name]['translator'].info()
 
        flow[name]['set'] = set_lang
        flow[name]['info'] = info

        is_set = False
        cookie = Cookie.SimpleCookie()
        cookie_string = flow.http.environ.get('HTTP_COOKIE')
        if cookie_string:
            cookie.load(cookie_string)
            if cookie.has_key('lang'):
                flow[name].set(languages=[cookie['lang'].value])
                is_set = True
        if not is_set:
            # Set up the null translator, defaults to None
            flow[name].set(self.default_language)
        flow[name]['N_'] = flow[name]['gettext_noop'] = gettext_noop

        def ugettext(value):
            """Mark a string for translation. Returns the localized unicode
            string of value.
        
            Mark a string to be localized as follows::
        
                _('This should be in lots of languages')
            
            """
            return flow[name].translator.ugettext(value)

        def gettext(value):
            """Mark a string for translation. Returns the localized string of
            value.
        
            Mark a string to be localized as follows::
        
                gettext('This should be in lots of languages')
        
            """
            return flow[name].translator.gettext(value)

        flow[name]['gettext'] = gettext
        flow[name]['_'] = flow[name]['ugettext'] = ugettext

        def ngettext(singular, plural, n):
            """Mark a string for translation. Returns the localized string of
            the pluralized value.
        
            This does a plural-forms lookup of a message id. ``singular`` is
            used as the message id for purposes of lookup in the catalog, while
            ``n`` is used to determine which plural form to use. The returned
            message is a string.
        
            Mark a string to be localized as follows::
        
                ngettext('There is %(num)d file here', 'There are %(num)d files here',
                         n) % {'num': n}
        
            """
            return flow[name].translator.ngettext(singular, plural, n)
        
        def ungettext(singular, plural, n):
            """Mark a string for translation. Returns the localized unicode
            string of the pluralized value.
        
            This does a plural-forms lookup of a message id. ``singular`` is
            used as the message id for purposes of lookup in the catalog, while
            ``n`` is used to determine which plural form to use. The returned
            message is a Unicode string.
        
            Mark a string to be localized as follows::
        
                ungettext('There is %(num)d file here', 'There are %(num)d files here',
                          n) % {'num': n}
        
            """
            return flow[name].translator.ungettext(singular, plural, n)

        flow[name]['ngettext'] = ngettext
        flow[name]['ungettext'] = ungettext

        def add_fallback(lang, **kwargs):
            """Add a fallback language from which words not matched in other
            languages will be translated to.
        
            This fallback will be associated with the currently selected
            language -- that is, resetting the language via set_lang() resets
            the current fallbacks.
        
            This function can be called multiple times to add multiple
            fallbacks.
            """
            return flow[name].translator.add_fallback(_get_translator(lang, **kwargs))

        flow[name]['add_fallback'] = add_fallback



